/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.guice.module;

import net.lucubrators.honeycomb.core.bind.BinderProvider;
import net.lucubrators.honeycomb.guice.binder.GuicedBinderProvider;
import net.lucubrators.honeycomb.guice.constraint.GuicedContraintProvider;
import net.lucubrators.honeycomb.guice.controller.GuicedControllerProvider;
import net.lucubrators.honeycomb.xml.constraint.ConstraintProvider;
import net.lucubrators.honeycomb.xml.controller.ControllerProvider;

import com.google.inject.AbstractModule;
import com.google.inject.Key;
import com.google.inject.Singleton;

public abstract class GuicedHoneycombModule extends AbstractModule {

	private final GuicedContraintProvider	contraintProvider;
	private final GuicedBinderProvider		binderProvider;
	private final GuicedControllerProvider	controllerProvider;

	public GuicedHoneycombModule(GuicedControllerProvider controllerProvider, GuicedBinderProvider binderProvider, GuicedContraintProvider contraintProvider) {
		this.controllerProvider = controllerProvider;
		this.binderProvider = binderProvider;
		this.contraintProvider = contraintProvider;
	}

	@Override
	protected void configure() {
		bind(ControllerProvider.class).toInstance(controllerProvider);
		bind(BinderProvider.class).toInstance(binderProvider);
		bind(ConstraintProvider.class).toInstance(contraintProvider);
		configureHoneycomb();
	}

	public ProviderMapper map(String id) {
		return new ProviderMapper(id);
	}

	public abstract void configureHoneycomb();

	public final class ProviderMapper {

		private String	id;

		public ProviderMapper(String id) {
			this.id = id;
		}

		public void toBinder(Class<?> binder) {
			Key<?> key = Key.get(binder);
			binderProvider.addBinder(id, key);
		}

		public void toConstraint(Class<?> constraint) {
			Key<?> key = Key.get(constraint);
			contraintProvider.addConstraint(id, key);
		}

		public void toController(Class<?> controller) {
			Key<?> key = Key.get(controller);
			controllerProvider.addController(id, key);
		}
	}

}
