/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.guice.web.filter;

import java.io.File;

import net.lucubrators.honeycomb.core.bind.BinderProvider;
import net.lucubrators.honeycomb.core.engine.BindablesFactory;
import net.lucubrators.honeycomb.core.engine.SubmittingCombResolver;
import net.lucubrators.honeycomb.defaultimpl.conversion.HoneycombConverter;
import net.lucubrators.honeycomb.defaultimpl.engine.ObjectInvoker;
import net.lucubrators.honeycomb.defaultimpl.web.filter.HoneycombFilter;
import net.lucubrators.honeycomb.defaultimpl.web.translator.RequestToCombIdTranslator;
import net.lucubrators.honeycomb.guice.binder.GuicedBinderProvider;
import net.lucubrators.honeycomb.guice.constraint.GuicedContraintProvider;
import net.lucubrators.honeycomb.guice.controller.GuicedControllerProvider;
import net.lucubrators.honeycomb.guice.web.listener.GuicedHoneycombContextListener;
import net.lucubrators.honeycomb.xml.constraint.ConstraintProvider;
import net.lucubrators.honeycomb.xml.controller.ControllerProvider;
import net.lucubrators.honeycomb.xml.registry.CombRegistry;

import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Singleton;
import com.google.inject.name.Named;

@Singleton
public class GuicedHoneycombFilter extends HoneycombFilter {

	public static final String	NAMED_CHAIN_ON_NO_COMB	= "guiced_chainOnNoComb";
	public static final String	NAMED_DEFINITION_FILE	= "guiced_honeycomb.xml";

	@Inject
	@Named(NAMED_DEFINITION_FILE)
	private File				definitionFile;

	@Override
	public void customSetup() {

		Injector guice = GuicedHoneycombContextListener.guice();

		ConstraintProvider constraintProvider = getConstraintProvider();
		ControllerProvider controllerProvider = getControllerProvider();
		BinderProvider binderProvider = getBinderProvider();
		if (constraintProvider != null && constraintProvider instanceof GuicedContraintProvider) {
			((GuicedContraintProvider) constraintProvider).init(guice);
		}
		if (controllerProvider != null && controllerProvider instanceof GuicedControllerProvider) {
			((GuicedControllerProvider) controllerProvider).init(guice);
		}
		if (binderProvider != null && binderProvider instanceof GuicedBinderProvider) {
			((GuicedBinderProvider) binderProvider).init(guice);
		}
	}

	@Override
	public File getDefinitionFile() {
		return definitionFile;
	}

	@Override
	public void addControllers() {
	}

	@Override
	public void addBinders() {
	}

	@Override
	public void addConstraints() {
	}

	@Inject(optional = true)
	@Override
	public void setBinderProvider(BinderProvider binderProvider) {
		super.setBinderProvider(binderProvider);
	}

	@Inject(optional = true)
	@Override
	public void setControllerProvider(ControllerProvider controllerProvider) {
		super.setControllerProvider(controllerProvider);
	}

	@Inject(optional = true)
	@Override
	public void setConstraintProvider(ConstraintProvider constraintProvider) {
		super.setConstraintProvider(constraintProvider);
	}

	@Inject(optional = true)
	@Override
	public void setBindablesFactory(BindablesFactory bindablesFactory) {
		super.setBindablesFactory(bindablesFactory);
	}

	@Inject(optional = true)
	@Override
	public void setCombRegistry(CombRegistry combRegistry) {
		super.setCombRegistry(combRegistry);
	}

	@Inject(optional = true)
	@Override
	public void setHoneycombConverter(HoneycombConverter honeycombConverter) {
		super.setHoneycombConverter(honeycombConverter);
	}

	@Inject(optional = true)
	@Override
	public void setObjectInvoker(ObjectInvoker objectInvoker) {
		super.setObjectInvoker(objectInvoker);
	}

	@Inject(optional = true)
	@Override
	public void setRequestToCombIdTranslator(RequestToCombIdTranslator requestToCombIdTranslator) {
		super.setRequestToCombIdTranslator(requestToCombIdTranslator);
	}

	@Inject(optional = true)
	@Override
	public void setSubmittingCombResolver(SubmittingCombResolver submittingCombResolver) {
		super.setSubmittingCombResolver(submittingCombResolver);
	}

	@Inject(optional = true)
	@Override
	public void setChainOnNoComb(@Named(NAMED_CHAIN_ON_NO_COMB) boolean chainOnNoComb) {
		super.setChainOnNoComb(chainOnNoComb);
	}

}
